var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Component } from '@angular/core';
import { Event, EventColumns, EventsDefaultSort, Attribute, AttributeColumns, AttributesDefaultSort } from '../../utilities/demoUtilities';
export let TagDemo = class TagDemo {
    constructor() {
        this.codeExample1 = `&lt;tagset&gt;
    &lt;tag *ngFor=&quot;#theTag of tags&quot;
        [color]=&quot;theTag.color&quot;
        [pill]=&quot;theTag.pill&quot;
        [removable]=&quot;theTag.removable&quot;
        [title]=&quot;theTag.title&quot;
        [value]=&quot;theTag.value&quot;
        (remove)=&quot;removeLog($event)&quot;&gt;
    &lt;/tag&gt;
&lt;/tagset&gt;`;
        this.codeExample2 = `export class TagExample {
    tags:any[] = [
        {title: 'Default'},
        {title: 'Primary', color: 'primary', pill: true, removable: true, value: 'Some great value'},
        {title: 'Info', color: 'info', pill: true, removable: true, value: false},
        {title: 'Success', color: 'success', pill: true, removable: true, value: 1234567890},
        {title: 'Danger', color: 'danger', pill: false, removable: true, value: {some: 'great', value: true}},
        {title: 'Warning', color: 'warning', pill: false, removable: true, value: true}
    ];
    
    removeLog(tag: Tag):void {
        console.log('Removed:', tag.title, '-', tag.value);
    }
}`;
        this.valueError = false;
        this.tempTag = {
            title: 'Example Title',
            color: 'default',
            disabled: false,
            pill: true,
            removable: true,
            value: 'Some value here'
        };
        this.tags = [
            { title: 'Default' },
            { title: 'Primary', color: 'primary', pill: true, removable: true, value: 'Some great value' },
            { title: 'Info', color: 'info', pill: true, removable: true, value: false },
            { title: 'Success', color: 'success', pill: true, removable: true, value: 1234567890 },
            { title: 'Danger', color: 'danger', pill: false, removable: true, value: { some: 'great', value: true } },
            { title: 'Warning', color: 'warning', pill: false, removable: true, value: true }
        ];
        this.tagAttributes = [
            new Attribute('color', 'string', 'default', 'Bootstrap color of the Tag. <a href="http://v4-alpha.getbootstrap.com/components/label/#contextual-variations" target="_blank">Click here for more info...</a>'),
            new Attribute('pill', 'boolean', 'false', 'Show the Tag as a pill'),
            new Attribute('title', 'string', 'null', 'Html of Tag\'s title'),
            new Attribute('value', 'any', 'null', 'Value of the Tag'),
            new Attribute('removable', 'boolean', 'false', 'Makes the tag able to be removed from the TagSet and adds an X icon to the title that removes the tag on click')
        ];
        this.tagAttributesColumns = AttributeColumns;
        this.tagAttributesSort = AttributesDefaultSort;
        this.tagEvents = [
            new Event('remove', '$event = tag: Tag', 'Returns the Tag object when removed')
        ];
        this.tagEventsColumns = EventColumns;
        this.tagEventsSort = EventsDefaultSort;
    }
    clearTags() {
        this.tags = [];
    }
    addTag() {
        this.valueError = false;
        this.tags.forEach((obj, i) => {
            if (this.tempTag.value === obj.value) {
                this.valueError = true;
            }
        });
        if (this.valueError)
            return;
        this.tags.push(JSON.parse(JSON.stringify(this.tempTag))); //make simple copy of tempTag to add
    }
    removeLog(tag) {
        console.log('Removed:', tag.title, '-', tag.value);
        try {
            this.tags.forEach((obj, i) => {
                if (tag.value === obj.value) {
                    throw this.tags.splice(i, 1);
                }
            });
        }
        catch (e) {
        }
    }
};
TagDemo = __decorate([
    Component({
        template: "\n    <div class=\"row\"><div class=\"col-md-12\"><div class=\"card card-block\"><h2 class=\"card-title\">Tags</h2><p class=\"card-text\">TagSet is a custom component to display a list of Tags</p></div></div></div><section class=\"row m-a\"><form><div class=\"form-group row\"><label for=\"tempTag.color\" class=\"col-sm-2 col-md-1 form-control-label\">Color</label><div class=\"col-sm-2\"><select class=\"custom-select\" [(ngModel)]=\"tempTag.color\" name=\"tempTag.color\"><option [selected]=\"tempTag.color == 'danger'\" value=\"danger\">Danger</option><option [selected]=\"tempTag.color == 'default'\" value=\"default\">Default</option><option [selected]=\"tempTag.color == 'info'\" value=\"info\">Info</option><option [selected]=\"tempTag.color == 'primary'\" value=\"primary\">Primary</option><option [selected]=\"tempTag.color == 'success'\" value=\"success\">Success</option><option [selected]=\"tempTag.color == 'warning'\" value=\"warning\">Warning</option></select></div><label for=\"tempTag.pill\" class=\"form-control-label\"><input #pillcb type=\"checkbox\" (change)=\"tempTag.pill = pillcb.checked\" [checked]=\"tempTag.pill\"> Pill</label><label for=\"tempTag.removable\" class=\"form-control-label\"><input #removablecb type=\"checkbox\" (change)=\"tempTag.removable = removablecb.checked\" [checked]=\"tempTag.removable\"> Removable</label><label for=\"tempTag.disabled\" class=\"form-control-label\"><input #disabledcb type=\"checkbox\" (change)=\"tempTag.disabled = disabledcb.checked\" [checked]=\"tempTag.disabled\"> Disabled</label></div><div class=\"form-group row\"><label for=\"tempTag.title\" class=\"col-sm-2 col-md-1 form-control-label\">Title</label><div class=\"col-sm-2\"><input class=\"form-control\" [(ngModel)]=\"tempTag.title\" name=\"tempTag.title\"></div><label for=\"tempTag.value\" class=\"col-sm-2 col-md-1 form-control-label\">Value</label><div class=\"col-sm-4\"><input class=\"form-control\" [(ngModel)]=\"tempTag.value\" name=\"tempTag.value\" ngControl=\"value\"><div [hidden]=\"!valueError\" class=\"alert alert-danger\">Value is already used for another tag</div></div><button type=\"submit\" class=\"btn btn-primary\" (click)=\"addTag()\">Add tag</button></div></form><p><button type=\"button\" class=\"btn btn-primary btn-sm\" [disabled]=\"tags.length <= 1\" (click)=\"tags[1].disabled = !tags[1].disabled\">Enable/Disable second tag</button> <button type=\"button\" class=\"btn btn-primary btn-sm\" [disabled]=\"tags.length == 0\" (click)=\"clearTags()\">Clear tags</button></p><div (click)=\"$event.preventDefault()\"><h3><tagset><tag *ngFor=\"let theTag of tags\" [color]=\"theTag.color\" [pill]=\"theTag.pill\" [disabled]=\"theTag.disabled\" [removable]=\"theTag.removable\" [title]=\"theTag.title\" [value]=\"theTag.value\" (remove)=\"removeLog($event)\"></tag></tagset></h3><h4>Tags</h4><div *ngFor=\"let theTag of tags\">{{theTag | json}}</div></div></section><div class=\"source\"><h3>Import</h3><pre>\n    <code class=\"language-javascript\" code-highlight>\n    {{'import {TAG_PROVIDERS} from \"fuel-ui/fuel-ui\"'}}\n    </code>\n    </pre><h3>Getting Started</h3><p>TagSet is a custom element to show an interactive tag interface. Used in conjuction with the custom Tag element. Tags can be displayed in a number ways</p><h3>Usage</h3><tabset><tab heading=\"HTML\"><pre>\n    <code class=\"language-markup\" code-highlight [innerHTML]=\"codeExample1\">\n    </code>\n    </pre></tab><tab heading=\"TypeScript\"><pre>\n    <code class=\"language-javascript\" code-highlight [innerHTML]=\"codeExample2\">\n    </code>\n    </pre></tab></tabset><h3>Tag Attributes</h3><table-sortable [columns]=\"tagAttributesColumns\" [data]=\"tagAttributes\" [sort]=\"tagAttributesSort\">Loading table...</table-sortable><h3>Tag Events</h3><table-sortable [columns]=\"tagEventsColumns\" [data]=\"tagEvents\" [sort]=\"tagEventsSort\">Loading table...</table-sortable></div>\n  "
    }), 
    __metadata('design:paramtypes', [])
], TagDemo);
